#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <utmp.h>
#include <unistd.h>

#include "error.h"
#include "utils.h"
#include "proc.h"

char *format_ts = "%Y-%m-%d %H:%M:%S";
char *format_load = "load: %f, %f, %f";
char *dir_proc = "/proc";

proc_t *prev_proc = NULL;
int n_prev_proc = 0;

char buffer[4096];

void emit_ts(void)
{
	time_t now = time(NULL);
	struct tm *tm_buffer = localtime(&now);

	if (strftime(buffer, sizeof(buffer), format_ts, tm_buffer) == 0)
		error_exit("emit_ts: strftime() failed\n");

	printf("%s\n", buffer);
}

void emit_load(void)
{
	double load[3];

	if (getloadavg(load, 3) == -1)
		error_exit("emit_load: getloadavg() failed\n");

	snprintf(buffer, sizeof(buffer), format_load, load[0], load[1], load[2]);

	printf("%s\n", buffer);
}

void emit_meminfo(void)
{
	char *data = load_file(dir_proc, "meminfo");
	char *line;
	long long memtotal, memfree, buffers, cached, swaptotal, swapfree, swapcached;
	long long memused, swapused;

	line = find_line(data, "MemTotal:");
	memtotal = get_value_field(line, 2);
	free(line);

	line = find_line(data, "MemFree:");
	memfree = get_value_field(line, 2);
	free(line);

	line = find_line(data, "Buffers:");
	buffers = get_value_field(line, 2);
	free(line);

	line = find_line(data, "Cached:");
	cached = get_value_field(line, 2);
	free(line);

	line = find_line(data, "SwapTotal:");
	swaptotal = get_value_field(line, 2);
	free(line);

	line = find_line(data, "SwapFree:");
	swapfree = get_value_field(line, 2);
	free(line);

	line = find_line(data, "SwapCached:");
	swapcached = get_value_field(line, 2);
	free(line);

	memused = memtotal - memfree;
	swapused = swaptotal - swapfree;
	cached += swapcached;

	/* FIXME: userdefined format */
	printf("MemTotal: %lld, MemUsed: %lld, MemFree: %lld, Buffers: %lld\n"
	       "SwapTotal: %lld, SwapUsed: %lld, SwapFree: %lld, Cached: %lld\n",
	       memtotal, memused, memfree, buffers,
	       swaptotal, swapused, swapfree, cached);

	free(data);
}

void emit_n_users_and_uptime(void)
{
	char *data = load_file(dir_proc, "uptime");
	unsigned long int uptime = get_value_field(data, 1);
	int years, weeks, days, hours, minutes, seconds;

	years = uptime / (86400 * 365.25);
	uptime -= years * 86400 * 365.25;

	weeks = uptime / (86400 * 7);
	uptime -= weeks * 86400 * 7;

	days = uptime / 86400;
	uptime -= days * 86400;

	hours = uptime / 3600;
	uptime -= hours * 3600;

	minutes = uptime / 60;
	uptime -= minutes * 60;

	seconds = uptime;

	free(data);

	printf("uptime: %d year, %d weeks, %d days, %d hours, %d minutes, %d seconds\n",
		years, weeks, days, hours, minutes, seconds);
}

void emit_proc_usage_and_mem_usage(int interval)
{
	proc_t *cur_proc = NULL;
	int n_cur_proc = 0, loop;
	unsigned long int interval_in_ticks = interval * sysconf(_SC_CLK_TCK);

	load_proc(dir_proc, &cur_proc, &n_cur_proc);

	// FIXME
	if (prev_proc)
	{
		for(loop=0; loop<n_cur_proc; loop++)
		{
			proc_t *prev_state = NULL;
			unsigned long int u_time_diff = 0, s_time_diff = 0;
			unsigned long int total_time_diff;
			double percentage;

			prev_state = find_proc_by_pid(prev_proc, n_prev_proc, cur_proc[loop].f_stat.pid);

			if (prev_state)
			{
				u_time_diff = cur_proc[loop].f_stat.utime - prev_state -> f_stat.utime;
				s_time_diff = cur_proc[loop].f_stat.stime - prev_state -> f_stat.stime;
			}
			else
			{
				u_time_diff = cur_proc[loop].f_stat.utime;
				s_time_diff = cur_proc[loop].f_stat.stime;
			}

			total_time_diff = u_time_diff + s_time_diff;

			percentage = ((double)total_time_diff * 100.0) / (double)interval_in_ticks;

			if (total_time_diff > 0)
			{
				printf("%5d %6.2f%% %c %s\n", cur_proc[loop].f_stat.pid, percentage, cur_proc[loop].f_stat.state, cur_proc[loop].cmdline);
				/* FIXME: limit length of cmdline to terminal-width */
				/* 26673 folkert   15   0 28212 7672 2656 S  0.0  0.8   0:42.76 btdownloadcurse */
			}
		}
	}

	free_proc_list(&prev_proc, &n_prev_proc);
	prev_proc = cur_proc;
	n_prev_proc = n_cur_proc;
}
