#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>

#include "error.h"
#include "proc.h"
#include "utils.h"

int load_proc_stat(char *path_name, proc_t *top)
{
	FILE *fh;
	char file_name[4096]; /* FIXME: path_max */
	char *dummy;

	snprintf(file_name, sizeof(file_name), "%s/stat", path_name);

	fh = fopen(file_name, "rb");
	if (!fh)
	{
		if (errno == ENOENT)
			return -1;

		error_exit("load_proc_stat: error loading file '%s'\n", file_name);
	}

        if (fscanf(fh, "%d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu %ld %ld %ld %ld %d 0 %llu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %d %lu %lu %llu",
                &top -> f_stat.pid,
                top -> f_stat.comm,
                &top -> f_stat.state,
                &top -> f_stat.ppid,
                &top -> f_stat.pgrp,
                &top -> f_stat.session,
                &top -> f_stat.tty_nr,
                &top -> f_stat.tpgid,
                &top -> f_stat.flags,
                &top -> f_stat.minflt,
                &top -> f_stat.cminflt,
                &top -> f_stat.majflt,
                &top -> f_stat.cmajflt,
                &top -> f_stat.utime,
                &top -> f_stat.stime,
                &top -> f_stat.cutime,
                &top -> f_stat.cstime,
                &top -> f_stat.priority,
                &top -> f_stat.nice,
                &top -> f_stat.itrealvalue,
                &top -> f_stat.starttime,
                &top -> f_stat.vsize,		/* 24 */
                &top -> f_stat.rss,
                &top -> f_stat.rsslim,
                &top -> f_stat.startcode,
                &top -> f_stat.endcode,
                &top -> f_stat.startstack,
                &top -> f_stat.kstkesp,
                &top -> f_stat.kstkeip,
                &top -> f_stat.signal,
                &top -> f_stat.blocked,
                &top -> f_stat.sigignore,
                &top -> f_stat.sigcatch,
                &top -> f_stat.wchan,
                &top -> f_stat.nswap,
                &top -> f_stat.cnswap,
                &top -> f_stat.exit_signal,
                &top -> f_stat.processor,
                &top -> f_stat.rt_priority,
                &top -> f_stat.policy,
                &top -> f_stat.blkio_tics) == EOF)
	{
		fclose(fh);
		return -1;
	}

	memmove(top -> f_stat.comm, top -> f_stat.comm + 1, sizeof(top -> f_stat.comm) - 1);
	dummy = strchr(top -> f_stat.comm, ')');
	if (dummy)
		*dummy = 0x00;

	fclose(fh);

	return 0;
}

int load_proc_cmdline(char *path_name, proc_t *proc)
{
	FILE *fh;
	char file_name[4096]; /* FIXME: path_max */

	snprintf(file_name, sizeof(file_name), "%s/cmdline", path_name);

	fh = fopen(file_name, "rb");
	if (!fh)
	{
		if (errno == ENOENT)
			return -1;

		error_exit("load_proc_cmdline: error loading file '%s'\n", file_name);
	}

	if (fgets(proc -> cmdline, sizeof(proc -> cmdline), fh) == NULL)
	{
		fclose(fh);
		return -1;
	}

	fclose(fh);

	return 0;
}

void load_proc(char *proc_dir, proc_t **pproc, int *n_proc)
{
	DIR *dir;
	struct dirent *de;

	*pproc = NULL;
	*n_proc = 0;

	if (chdir(proc_dir) == -1)
		error_exit("load_proc: chdir(%s) failed\n", proc_dir);

	dir = opendir(proc_dir);
	if (!dir)
		error_exit("load_proc: opendir(%s) failed\n", proc_dir);

	while((de = readdir(dir)) != NULL)
	{
		struct stat statbuf;
		int loop, name_len = strlen(de -> d_name);
		char is_digit_name = 1;
		char path_name[4096]; /* FIXME: path_max */

		/* procs are digit-only dir-entries */
		for(loop=0; loop<name_len; loop++)
		{
			if (!isdigit((de -> d_name)[loop]))
			{
				is_digit_name = 0;
				break;
			}
		}
		if (!is_digit_name)
			continue;

		if (stat(de -> d_name, &statbuf) == -1)
		{
			/* proc went away before we're able to process it */
			if (errno == ENOENT)
				continue;

			error_exit("load_proc: stat(%s/%s) failed\n", proc_dir, de -> d_name);
		}

		/* must be a directory */
		if (!S_ISDIR(statbuf.st_mode))
			continue;

		/* grow aray containing data */
		*pproc = (proc_t *)realloc(*pproc, sizeof(proc_t) * (*n_proc + 1));
		if (!*pproc)
			error_exit("load_proc: failed to reallocate proc data-structure to %d elements\n", *n_proc + 1);
		memset(&(*pproc)[*n_proc], 0x00, sizeof(proc_t));

		snprintf(path_name, sizeof(path_name), "%s/%s", proc_dir, de -> d_name);

		/* load /proc/.../stat-data into structure */
		if (load_proc_stat(path_name, &(*pproc)[*n_proc]) == -1)
			continue;

		/* load /proc/.../cmdline-data into structure */
		if (load_proc_cmdline(path_name, &(*pproc)[*n_proc]) == -1)
			continue;

		(*n_proc)++;
	}

	closedir(dir);
}

proc_t * find_proc_by_pid(proc_t *proc_list, int n_proc, int pid)
{
	int loop;

	for(loop=0; loop<n_proc; loop++)
	{
		if (proc_list[loop].f_stat.pid == pid)
			return &proc_list[loop];
	}

	return NULL;
}

void free_proc_entry(proc_t *pproc)
{
}

void free_proc_list(proc_t **pproc, int *n_proc)
{
	int loop;

	for(loop=0; loop<*n_proc; loop++)
	{
		free_proc_entry(&(*pproc)[loop]);
	}

	free(*pproc);

	*pproc = NULL;
	*n_proc = 0;
}
