#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>

#include "main.h"
#include "error.h"

char *load_file(char *dir, char *file)
{
	int fd;
	char filename_buffer[4096]; /* FIXME: path_max */
	char *data_buffer = (char *)malloc(MAX_PROCFILE_SIZE + 1);
	size_t n_read;

	if (!data_buffer)
		error_exit("load_file: error allocating %d bytes of memory\n", MAX_PROCFILE_SIZE);

	snprintf(filename_buffer, sizeof(filename_buffer), "%s/%s", dir, file);

	fd = open(filename_buffer, O_RDONLY);
	if (fd == -1)
		error_exit("load_file: error accessing file %s\n", filename_buffer);

	n_read = read(fd, data_buffer, MAX_PROCFILE_SIZE);
	if (n_read <= 0)
		error_exit("load_file: error reading from file %s\n", filename_buffer);

	close(fd);

	data_buffer[n_read] = 0x00;

	return data_buffer;
}

char * find_line(char *in, char *what)
{
	char *new_str;
	char *start = strstr(in, what), *end;
	if (!start)
		error_exit("find_line: '%s' not found in input (%s)\n", what, in);

	while(start > in && *start != '\n')
		start--;
	if (*start == '\n')
		start++;

	new_str = strdup(start);
	if (!new_str)
		error_exit("find_line: error doing strdup on '%s'\n", start);

	end = strchr(new_str, '\n');
	if (end)
		*end = 0x00;

	return new_str;
}

long long get_value_field(char *line, int field_nr)
{
	for(;;)
	{
		char *field;

		/* find start of field */
		while(isspace(*line)) line++;
		field = line;

		/* find end of field */
		while(!isspace(*line) && *line != 0x00) line++;
		if (*line)
		{
			*line = 0x00;
			line++;
		}

		field_nr--;

		if (field_nr == 0)
			return atoll(field);
	}

	return -1;
}
